package insights

// Copyright (c) Microsoft and contributors.  All rights reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Code generated by Microsoft (R) AutoRest Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

import (
	"context"
	"github.com/Azure/go-autorest/autorest"
	"github.com/Azure/go-autorest/autorest/azure"
	"github.com/Azure/go-autorest/tracing"
	"net/http"
)

// GetClient is the composite Swagger for Application Insights Data Client
type GetClient struct {
	BaseClient
}

// NewGetClient creates an instance of the GetClient client.
func NewGetClient() GetClient {
	return NewGetClientWithBaseURI(DefaultBaseURI)
}

// NewGetClientWithBaseURI creates an instance of the GetClient client using a custom endpoint.  Use this when
// interacting with an Azure cloud that uses a non-standard base URI (sovereign clouds, Azure stack).
func NewGetClientWithBaseURI(baseURI string) GetClient {
	return GetClient{NewWithBaseURI(baseURI)}
}

// Metadata retrieve the metadata information for the app, including its schema, etc.
// Parameters:
// appID - ID of the application. This is Application ID from the API Access settings blade in the Azure
// portal.
func (client GetClient) Metadata(ctx context.Context, appID string) (result MetadataResults, err error) {
	if tracing.IsEnabled() {
		ctx = tracing.StartSpan(ctx, fqdn+"/GetClient.Metadata")
		defer func() {
			sc := -1
			if result.Response.Response != nil {
				sc = result.Response.Response.StatusCode
			}
			tracing.EndSpan(ctx, sc, err)
		}()
	}
	req, err := client.MetadataPreparer(ctx, appID)
	if err != nil {
		err = autorest.NewErrorWithError(err, "insights.GetClient", "Metadata", nil, "Failure preparing request")
		return
	}

	resp, err := client.MetadataSender(req)
	if err != nil {
		result.Response = autorest.Response{Response: resp}
		err = autorest.NewErrorWithError(err, "insights.GetClient", "Metadata", resp, "Failure sending request")
		return
	}

	result, err = client.MetadataResponder(resp)
	if err != nil {
		err = autorest.NewErrorWithError(err, "insights.GetClient", "Metadata", resp, "Failure responding to request")
	}

	return
}

// MetadataPreparer prepares the Metadata request.
func (client GetClient) MetadataPreparer(ctx context.Context, appID string) (*http.Request, error) {
	pathParameters := map[string]interface{}{
		"appId": autorest.Encode("path", appID),
	}

	preparer := autorest.CreatePreparer(
		autorest.AsGet(),
		autorest.WithBaseURL(client.BaseURI),
		autorest.WithPathParameters("/apps/{appId}/metadata", pathParameters))
	return preparer.Prepare((&http.Request{}).WithContext(ctx))
}

// MetadataSender sends the Metadata request. The method will close the
// http.Response Body if it receives an error.
func (client GetClient) MetadataSender(req *http.Request) (*http.Response, error) {
	return client.Send(req, autorest.DoRetryForStatusCodes(client.RetryAttempts, client.RetryDuration, autorest.StatusCodesForRetry...))
}

// MetadataResponder handles the response to the Metadata request. The method always
// closes the http.Response Body.
func (client GetClient) MetadataResponder(resp *http.Response) (result MetadataResults, err error) {
	err = autorest.Respond(
		resp,
		client.ByInspecting(),
		azure.WithErrorUnlessStatusCode(http.StatusOK),
		autorest.ByUnmarshallingJSON(&result),
		autorest.ByClosing())
	result.Response = autorest.Response{Response: resp}
	return
}
